/* file: "main.c" */

/* Copyright (C) 1994-2002 by Marc Feeley, All Rights Reserved. */

/* This is the driver of the Gambit-C system */

#define ___INCLUDED_FROM_MAIN
#define ___VERSION 40009
#include "gambit.h"

#include "os.h"


___HIDDEN void usage_err
   ___P((char **argv,
         int debug_level),
        (argv,
         debug_level)
char **argv;
int debug_level;)
{
  if (debug_level > 0)
    {
      char *msgs[4];
      msgs[0] = "Usage: ";
      msgs[1] = argv[0];
      msgs[2] = " [-:OPT,OPT...] ...\n"
                "  where OPT can be one of:\n"
                "    mMIN_HEAP_SIZE_IN_KILOBYTES\n"
                "    hMAX_HEAP_SIZE_IN_KILOBYTES\n"
                "    lPERCENT_LIVE_AFTER_GC\n"
                "    s[level]\n"
                "    d[level]\n";
      msgs[3] = 0;
      ___display_error (msgs);
    }
  ___exit_process (1);
}


___EXP_FUNC(int,___main)
  ___P((int argc,
        char **argv,
        ___mod_or_lnk (*linker)(___global_state_struct*)),
       (argc,
        argv,
        linker)
int argc;
char **argv;
___mod_or_lnk (*linker)();)
{
#define LARGEST_INT (int)((~(unsigned int)0)>>1)

  int options_source;
  long min_heap_len;
  long max_heap_len;
  int live_percent;
  int standard_level;
  int debug_level;
  ___ERR_CODE err;
  ___setup_params_struct setup_params;

  /* handle arguments to runtime */

  min_heap_len = 0;
  max_heap_len = 0;
  live_percent = 0;
  standard_level = 0;
  debug_level = 1;

  for (options_source=0; options_source<2; options_source++)
    {
      char *arg;

      if (options_source == 0)
        arg = ___getenv (___GAMBCOPT_ENV_VAR);
      else
        {
          arg = argv[1];
          if (argc > 1 && *arg++ == '-' && *arg++ == ':')
            {
              argv[1] = argv[0];
              argc--;
              argv++;
            }
          else
            arg = 0;
        }

      if (arg == 0)
        continue;

      do
        {
          char *s = arg++;
          switch (*s)
            {
            case 'm':
            case 'h':
            case 'l':
            case 'd':
            case 's':
              {
                int argval = 0;
                while (*arg >= '0' && *arg <= '9')
                  {
                    int n = *arg - '0';
                    if (argval > (LARGEST_INT>>10)/10 ||
                        (argval == (LARGEST_INT>>10)/10 &&
                         n > ((LARGEST_INT>>10)-argval*10)))
                      usage_err (argv, debug_level);
                    argval = argval*10 + n;
                    arg++;
                  }
                if (arg == s+1 && *s == 's')
                  argval = 5;
                else if (arg == s+1 && *s == 'd')
                  argval = 1;
                else if (arg == s+1)
                  usage_err (argv, debug_level);
                else
                  argval = atoi (s+1);
                switch (*s)
                  {
                  case 'm': min_heap_len = argval<<10;
                            break;
                  case 'h': max_heap_len = argval<<10;
                            break;
                  case 'l': live_percent = argval;
                            break;
                  case 's': standard_level = argval;
                            break;
                  case 'd': debug_level = argval;
                            break;
                  }
                break;
              }
            default:
              usage_err (argv, debug_level);
            }
        }
      while (*arg++ == ',');
      if (*--arg != '\0')
        usage_err (argv, debug_level);
    }

#ifdef FORCE_MAX_HEAP
  if (max_heap_len == 0 || max_heap_len > FORCE_MAX_HEAP<<10)
    max_heap_len = FORCE_MAX_HEAP<<10;
#endif

  /* Setup program, run it and perform any cleanup necessary. */

  ___setup_params_reset (&setup_params);

  setup_params.version        = ___VERSION;
  setup_params.argv           = argv;
  setup_params.min_heap       = min_heap_len;
  setup_params.max_heap       = max_heap_len;
  setup_params.live_percent   = live_percent;
  setup_params.standard_level = standard_level;
  setup_params.debug_level    = debug_level;
  setup_params.linker         = linker;

  err = ___setup (&setup_params);

  if (err != ___NO_ERR)
    {
      char *msgs[2];
      char msg[___ERR_MAX_LENGTH+1];
      ___err_code_to_string (err, msg);
      msgs[0] = msg;
      msgs[1] = 0;
      ___fatal_error (msgs);
    }

  /* Cleanup the program and exit */

  ___cleanup_and_exit_process (0);

  return 0; /* satisfy some compilers */
}
